//---------------------------------------------------------------------------

#ifndef stolickaH
#define stolickaH

//#include <math.h>
#include "strom.h"
#include "konvex.h"
//---------------------------------------------------------------------------

/**
 * trieda na pracu so stolickou
 */
class stolicka
{
 public:
  /**
   * poradove cislo stolicky
   */
  int cislo;

  /**
   * datova struktura popisujuca hodnoty fitness, ktore dostala tato stolicka za jednotlive kriteria
   */
  hodnoty_fitness hodn_fit;  //hodnoty fitness za jednotlive kriteria

  /**
   * datova struktura popisujuca nastavnie, akym sposobom sa ma vyhodnocovat fitness funkcia stolicky
   */
  nastavenie_fitness nast;   //navod, ako sa ma vyhodnocovat fitness funkcia tejto stolicky

  /**
   * true, ak je na vrchu utvaru polozena doska
   */
  bool stol;                 //ci ma byt navrchu polozena doska ako na stole

 private:
  /**
   * hodnota fitness stolicky
   */
  float fit;        //fitness

  //asi nanic
  float stabilita;  //relativna stabilita stolicky

 public:
  /**
   * strom, v ktorom su ulozene data, v ktorych je zakodovany tvar stolicky
   */
  strom *posuny;

  /**
   * maximalna absolutna suradnica v smere osi x, z
   */
  float mox,moz;      //maximalna absolutna suradnica v smere osi x, z

  /**
   * maximalna y-ova suradnica, vyska najvyssieho body v stolicke
   */
  float maxy;         //maximalna  y-ova suradnica v stolicke (najvyssi bod stolicky)

  /**
   * minimalna y-ova suradnica, vyska najnizsieho bodu v stolicke
   */
  float miny;         //minimalnay-ova suradnica stolicky (najnizsi bod stolicky)

  /**
   * minimalne a maximalne hodnoty suradnic v stolicke
   */
  float minz,maxz;

  /**
   * true, ak je stolicka zrkadlovo symetricka (podla roviny z-x)
   */
  bool symetria;    //ci ma byt stolicka symetricka (zrkadlovo podla roviny z-x)

  /**
   * true, ak je stolicka zrkadlovo syemtricka podla dvoch rovin (roviny x-z a x-y)
   */
  bool symetria4;   //symetria podla roviny x-z a x-y

  /**
   * pocet useciek v stolicke
   */
  int pocet_useciek;  //pocet useciek v stolicke

 private:
  /**
   * pocet vsetkych bodov stolicky
   */
  int pocet_bodov;    //pocet bodov celej stolicky

  /**
   * najvysii bod stolicky
   */
  bod maxy_bod;       //najvyssi bod stolicky

 public:
  /**
   * datova struktura, v ktorej sa nachadza v kazdom riadku (t.j.\ utvary[i]) suradnice jedneho utvaru - jednej plochy alebo jednej ciary
   */
  bod utvary[100][30];     //suradnice jedneho utvaru, t.j. jednej plochy alebo jednej ciary

  /**
   * dlzky zaznamov v poli utvary, ak je dlzky[i]<100, tak je to ciara, ak je dlzky[i]>100, tak je to rovina v dlzke dlzky[i]-100
   */
  int dlzky[100];          //dlzky zaznamov v poli utvary, ak je dlzky[i]<100, tak je to ciara
                           //                              ak je dlzky[i]>100, tak je to rovina v dlzke dlzky[i]-100
 public:
  /**
   * pole bodov, ktore su opornymi bodmi stoliky, body, ktore sa nachadzaju na zemi
   */
  bod opor_body[100];      //oporne body - body, ktore su na zemi

  /**
   * pole indexov do pola utvary na riadky, v ktorych sa nachadzaju roviny, ktore su sedatkami
   */
  int sedatka[100];        //index do pola utvary na riadky, v ktorych sa nachadza rovina, ktora je sedatkom

  /**
   * pocet prvkov v poli sedatka
   */
  int pocet_sedatok;       //pocet prvkov v poli sedatka;

 public:
  /**
   * pocet opornych bodov, pocet prvkov v poli opor_body
   */
  int  pocmin;   //pocet opornych bodov = pocet bodov, ktore nie su vyssie ako 1 od najnizsieho bodu

 private:
  /**
   * bod, v ktorom zacina stolicka, konkretne (0,0,0)
   */
  bod zaciatok;
  
 public:
  /**
   * Konstruktor.
   * vyrobi prazdnu stolicku bez dalsich nastaveni
   */
  stolicka(int uhol);                     //vyrobi prazdnu stolicku

  /**
   * Konstruktor.
   * vyrobi prazdnu stolicku a nastavi prislusne hodnoty pomocou nacitanych udajov
   * @param n urcuje, akym sposobom sa bude vyhodnocovat fitness stolicky
   * @param sym true, ak ma byt stolicka zrkadlovo symetricka
   * @param sym4 true, ak ma byt stolicka zrkadlovo symetricka podla dvoch rovin
   * @param doska true, ak ma byt navrchu polozena doska
   */
  stolicka(nastavenie_fitness n, bool sym, bool sym4, bool doska);

  /**
   * Destruktor.
   * vyprazdni strom, v ktorom je ulozena reprezentacia stolicky
   */
  ~stolicka();

 public:
  /**
   * Pocita bod, ktory je taziskom celej stolicky.
   * tazisko roviny sa pocita ako tazisko hran, ktore ju ohranicuju
   * @return bod, ktory je taziskom stolicky
   */
  bod tazisko();               //tazisko celej stolicky

  /**
   * Pocita tazisko jednej roviny, ktore ja ulozena v utvary[x]
   * @param x index do pola utvary, udava poziciu, na ktorej sa nachadza rovina, ktorej tazisko sa ma pocitat
   * @return tazisko roviny
   */
  bod tazisko_roviny(int x);   //tazisko roviny v utvary[x]

  /**
   *Ppocita tazisko opornych bodov.
   kedze tieto su v jednej rovine, pocita sa pomocou aritmetickeho priemeru prislusnych suradnic
   * @return tazisko opornych bodov
   */
  bod tazisko_opor_bodov();    //aritmeticky stred (priemer) opornych bodov

  /**
   * Naplni polia dlzky a utvary hodnotami, ktore dostane z obejktu strom
   */
  void vyrataj_body();

  /**
   * Postupne prechadza vsetky roviny v stolicke a zistuje, ktore z nich su sedatkami.
   * (t.j. priblizne vodorovnymi rovinami)
   * indexy tychto rovin v poli utvary da do pola sedatka
   */
  void najdi_sedatka();

  /**
   * Zistenie hodnoty fitness za kriteria pre sedatka
   * vyhodnocuje sa fitness za kriterium vysky a stability sedatka, 
   * vysledkom je maximalny sucet hodnot pre obidve kriteria, ktory dosiahlo jedno zo sedatok
   * @return hodnota fitness za sedatka v stolicke
   */
  float fitness_za_sedatka();

  /**
   * Znizuje najnizsie body stolicky na uroven najnizsieho.
   * znizi 3 alebo 2 najznisie body v stolicke na uroven najnizsieho body v stolicke, podla toho ci nie je symetricka (3) alebo je zrkadlovo symetricka (2)
   * ak je stolicka dvojnasobne symetricka, nerobi nic (nema to zmysel)
   */
  void zniz_body();            //znizi 3(2) najnizise body na uroven najnizsieho z nich

  /**
   * Urobi stolicku symetrickou.
   * t.j. prejde vsetky zaznamy a zduplikuje ich, pricom zmeni jednu suradnicu tak, 
   * aby novovzniknuty utvar bol zrkadlovo symetricky oproti povodnemu utavru
   */
  void urob_symetriu();        //urobi stolicku symetrickou

  /** 
   * Urobi stolicku dvojnasobne zrkadlovo symetrickou.
   * t.j. prejde vsetku zaznamy v stolicke a pre kazdy z nich vyrobi 3 dalsie zaznamy,
   * ktore su navzajoom zrkadlovo symetricke
   */
  void urob_4symetriu();       //urobi stolicku 4symetrickou
  
 public:
  /**
   * Zmutuje stolicku.
   */
  void mutuj();

  /**
   * Vyhodnocovanie fitness funkcie
   * @return hodnota fitness stolicky
   */
  float fitness();

  /**
   * Najde zoznam na krizenie a odoberie z neho nejaku cast
   * @return pointer na prvok, ktory je zaciatkom casti zoznamu, ktory sa dava k dispozicii na krizenie
   */
  prvok* daj_na_krizenie();

  /**
   * Prida na miesto, z ktoreho bola cast jedneho zo zoznamov, novu cast zoznamu
   * @param *uk pointer na prvok, ktory je prvym prvkom casti zoznamu, ktory sa ma doplnit na miesto, z ktoreho bol predtym odobraty kus zoznamu
   */
  void pridaj_pri_krizeni(prvok *uk);

  /**
   * Zistuje, ci sa v srolicke nachadza aspon jedno sedatko
   * @return true, ak sa v stolicke nachadza aspon jedno sedatko
   */
  bool je_sedatko();

  /**
   * Zistuje, ci je stolicka stabilna
   * @return true, ak je stolicka ako celok stabilna
   */
  bool stabilna();

  /**
   * Zduplikuje celu stolicku
   * @return pointer na presnu kopiu stolicky
   */
  stolicka* zduplikuj_sa();

  /**
   * Postupne prejde vsetku body v stolicke a najde oporne body, tieto body da do pola opor_body
   */
  void vyrataj_oporne_body();

  /**
   * Zisti dlzku stolicky. 
   * t.j. pocet zaznamov v poli utvary
   * @return pocet zaznamov v polie utvary
   */
  int length();

  /**
   * Ulozi stolicku do suboru
   * @param filename meno suboru, do ktoreho bude stolicka ulozena
   */
  void uloz_do_suboru(AnsiString filename);
  
  /**
   * Nacita data o stolicke zo suboru
   * @param filename meno suboru, z ktoreho budu udaje nacitanw
   */
  void nacitaj_zo_suboru(AnsiString filename);

  /** 
   * Strom prazdnej stolicky naplni nahodnymi udajmi - vytvori nahodnu stolicku
   */
  void napln_nahodne();

  /**
   * Zistuje, ci plocha v utvary[x] je sedatko.
   * @param x index do pola utvary, kde sa nachadza kontrolovana rovina
   * @return true, ak je tato plocha sedatkom
   */
  bool plocha_je_sedatko(int x);

};

/**
 * pocita vzdialenost bodov v priestore bez ohladu na y-ovy suradnicu
 * @param b1,b2 prvy bod
 * @param b2 druhy bod
 * @return vzdialenost danych dvoch bodov
 */
float vzdialenost_bodov_bez_y(bod b1, bod b2);

/**
 * pocita vzdialenost dvoch bodov v priestore
 * @param b1,b2 prvy bod
 * @param b2 druhy bod
 * @return vzdialenost danych dvoch bodov
 */
float vzdialenost_bodov(bod b1, bod b2);
    //dlazka je rovina x-z!!!
#endif

