//---------------------------------------------------------------------------

#ifndef zoznamH
#define zoznamH
//---------------------------------------------------------------------------

class zoznam;

/**
 * Struct posun definuje otocenie v priestore pre jednu usecku a dlzku usecky
 */
struct posun
{
  /**
   * Honoty parametrov
   */
  float x,y,z;
};

/**
 * Struct element popisuje kompletne udaje o jednej usecke a o rovine k nej pridruzenej
 */
struct element
{
  /**
   * Paramatre zakladnej usecky
   */
  posun pos;
  /**
   * Pocet bodov plochy, vysledna plocha ma plocha+2 vrcholov, tiez je to pocet prvkov v poli posuny_plochy a v poli nasledovnici
   */
  int plocha;   //pocet dalsich vrcholov plochy, ak plocha=0, tak nie je plocha
  /**
   * Pole useciek ohranicujucich plochu
   */
  posun posuny_plochy[10]; //posuny urcujuce plochu
  /**
   * Pole nasledovnikov, ktori mozu byt pripojeni v kazdom vrchole plochy, ktory nelezi na povodnej usecke
   */
  zoznam *nasledovnici[10]; //ako pokracuje utvar v bodoch roviny
};

/**
 * Struct prvok je prvok zoznamu
 */
struct prvok      //prvok zoznamu
{
  /**
   * Udaje o jednej usecke a o rovine k nej pridruzenej
   */
  element data;
  /**
   * Ukazovatel na nasledujuci prvok zoznamu
   */
  prvok *next;
};

/**
 * Trieda urcena na pracu so zoznamami
 */
class zoznam
{
 private:
  /**
   * Ukazovatel na prvy prvok zoznamu
   */
  prvok *zac;  //ukazovatel na prvy prvok zoznamu
  
  /**
   * Ukazovatel na aktualny prvok v zozname
   */
  prvok *akt;  //ukazovatel na aktualny prvok

  /**
   * Ukazovatel na posledny prvok zoznamu
   */
  prvok *kon;  //ukazovatel na posledny prvok

  /**
   * Dlzka zoznamu
   */
  int dlzka;

 public:
  /**
   * Konstruktor.
   * vyrobi prazdny zoznam
   */
  zoznam();

  /**
   * Destruktor.
   * ak zoznam nie je prazdny, tak ho vypradzni
   */
  ~zoznam();

  /**
   * Zistuje, ci je zoznam prazdny
   * @return vrati true, ak je zoznam prazdny
   */
  bool is_empty();

  /**
   * Prida prvok na koniec zoznamu
   * @param data udaje o jednej usecke
   */
  void push(element data);

  /**
   * Vyberie prvy prvok zo zoznamu
   * @return element popisujuci jednu usecku
   */
  element pop();

  /**
   * Nastavi aktualnu poziciu na zaciatok zoznamu
   */
  void reset();

  /**
   * Vrati dlzku zoznamu.
   * pocet prvkov v zozname
   * @return pocet prvkov zoznamu
   */
  int length();

  /**
   * Zistuje, ci je aktualna pozicia na konci zoznamu
   * @return true, ak je akt ukazuje za posledny prvok zoznamu (NULL)
   */
  bool is_end();   //ak je za zoznamom, tak true

  /**
   * Vrati hodnotu aktualneho prvku a posunie sa na nasledujuci prvok
   * @return hodnota aktualneho prvku
   */
  element return_value();

  /**
   * Vracia cast zoznamu, od nahodnej pozicie po koniec, povodny zoznam sa prislusne skrati
   * @return pointer na zaciatok casti povodneho zoznamu
   */
  prvok* kus_zoznamu();   

  /**
   * Prida na koniec zoznamu nejaku cast ineho zoznamu
   * @param pointer na prvy prvok pridavanej casti zoznamu
   */
  void pridaj_na_koniec(prvok *ukaz);

  /**
   * Vrati hodnotu aktualneho prvku bez toho, aby sa posunul na nasledujuci prvok
   * @return hodnota aktualneho prvku
   */
  element vrat_hodnotu_akt_prvku();

  /**
   * Meni hodnotu aktualneho prvku bez toho, aby sa posunul na nasledujucu poziciu
   * @param pos nova hodnota prvku
   */
  void zmen_akt_prvok(element pos);

  /**
   * Vracia pointer na prvy prvok zoznamu
   * @return pointer na prvy prvok zoznamu
   */
  prvok* zaciatok();
  
 private:
   /**
    * Nastavi pointer kon na koniec zoznamu a zisti novu dlzku zoznamu
    */
  void refresh();
};

#endif
